@@ -1,10 +1,13 @@
 import logging
 
+from django.contrib.auth import get_user_model
 from django.utils.translation import ugettext_lazy as _
 from rest_framework import serializers, exceptions
 
 from notification_system.models import (NotificationEvent, EmailTemplate, Attachment, SmtpProvider, NotificationGroup,
-                                        OutgoingMessage)
+                                        OutgoingMessage, Media)
+
+User = get_user_model()
 
 logger = logging.getLogger('notification_system')
 
@@ -21,6 +24,12 @@ class EmailTemplateSerializer(serializers.ModelSerializer):
         fields = ['id', 'title', 'subject', 'content']
 
 
+class MediaSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = Media
+        fields = ['id', 'file']
+
+
 class MinimalSmtpProviderSerializer(serializers.ModelSerializer):
     class Meta:
         model = SmtpProvider
@@ -28,9 +37,12 @@ class MinimalSmtpProviderSerializer(serializers.ModelSerializer):
 
 
 class MinimalNotificationGroupSerializer(serializers.ModelSerializer):
+    members = serializers.PrimaryKeyRelatedField(many=True, queryset=User.objects.filter(is_active=True),
+                                                 required=False)
+
     class Meta:
         model = NotificationGroup
-        fields = ['id', 'name']
+        fields = ['id', 'name', 'members']
 
 
 class NotificationEventSerializer(serializers.ModelSerializer):
@@ -38,34 +50,56 @@ class NotificationEventSerializer(serializers.ModelSerializer):
     template_detail = EmailTemplateSerializer(source='template', read_only=True)
     smtp_provider_detail = MinimalSmtpProviderSerializer(source='smtp_provider', read_only=True)
     notification_group_detail = MinimalNotificationGroupSerializer(source='notification_group', read_only=True)
+    recipients = serializers.PrimaryKeyRelatedField(many=True, queryset=User.objects.filter(is_active=True),
+                                                    required=False)
 
     class Meta:
         model = NotificationEvent
         fields = ['id', 'title', 'smtp_provider', 'smtp_provider_detail', 'template', 'template_detail', 'subject',
-                  'content', 'notification_group', 'notification_group_detail', 'attachments']
+                  'content', 'notification_group', 'notification_group_detail', 'attachments', 'recipients']
 
         extra_kwargs = {
-            'template': {'write_only': True},
+            'template': {'write_only': True, 'allow_null': True},
             'smtp_provider': {'write_only': True},
-            'notification_group': {'write_only': True},
+            'notification_group': {'write_only': True, 'allow_null': True},
         }
 
     def create(self, validated_data):
         attachments = validated_data.pop('attachments', list())
         instance = super().create(validated_data)
 
+        attachment_list = []
         for attachment in attachments:
-            instance.attachments.create(file=attachment.get('file'))
+            attachment_list.append(Attachment(file=attachment.get('file'), notification_event=instance))
+
+        if attachment_list:
+            instance.attachments.bulk_create(attachment_list)
+
+        return instance
+
+    def update(self, instance, validated_data):
+        attachments = validated_data.pop('attachments', list())
+        instance = super().update(instance, validated_data)
+
+        if attachments:
+            instance.attachments.all().delete()
+
+        attachment_list = []
+        for attachment in attachments:
+            attachment_list.append(Attachment(file=attachment.get('file'), notification_event=instance))
+
+        if attachment_list:
+            instance.attachments.bulk_create(attachment_list)
 
         return instance
 
     def validate_template(self, value):
-        if (not value.is_active) or (not self.context['request'].user == value.owner):
+        if value and ((not value.is_active) or (not self.context['request'].user == value.owner)):
             raise exceptions.ValidationError(detail=_("You do not have permission to access this template."))
         return value
 
     def validate_notification_group(self, value):
-        if (not value.is_active) or (not self.context['request'].user == value.owner):
+        if value and ((not value.is_active) or (not self.context['request'].user == value.owner)):
             raise exceptions.ValidationError(detail=_("You do not have permission to access this notification group."))
         return value
 
