@@ -11,9 +11,10 @@ from rest_framework_simplejwt.authentication import JWTAuthentication
 
 from notification_system.api.serializers import (NotificationEventSerializer, EmailTemplateSerializer,
                                                  OutgoingMessageSerializer, MinimalNotificationGroupSerializer,
-                                                 MinimalSmtpProviderSerializer, OutgoingMessageNotificationSerializer)
+                                                 MinimalSmtpProviderSerializer, OutgoingMessageNotificationSerializer,
+                                                 MediaSerializer)
 from notification_system.models import (NotificationEvent, EmailTemplate, OutgoingMessage, NotificationGroup,
-                                        SmtpProvider)
+                                        SmtpProvider, Media)
 from notification_system.notification_delivery import broadcast_event_messages_to_recipients
 from notification_system.paginations import CustomLimitOffsetPagination
 from notification_system.parsers import PlainTextParser
@@ -24,12 +25,12 @@ from notification_system.tasks import update_outgoing_message_status
 class NotificationEventViewSet(mixins.ListModelMixin,
                                mixins.RetrieveModelMixin,
                                mixins.CreateModelMixin,
+                               mixins.UpdateModelMixin,
                                viewsets.GenericViewSet):
     """
     A ViewSet for viewing Notification Event.
     """
-    queryset = NotificationEvent.actives.prefetch_related('attachments').select_related('smtp_provider', 'template',
-                                                                                        'notification_group').all()
+    queryset = NotificationEvent.actives.all()
     serializer_class = NotificationEventSerializer
     authentication_classes = [JWTAuthentication]
     permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
@@ -38,7 +39,10 @@ class NotificationEventViewSet(mixins.ListModelMixin,
     ordering_fields = ['id']
 
     def get_queryset(self):
-        return super().get_queryset().filter(owner=self.request.user)
+        queryset = super().get_queryset()
+        queryset = queryset.prefetch_related('attachments', 'notification_group__members', 'recipients')
+        queryset = queryset.select_related('smtp_provider', 'template', 'notification_group')
+        return queryset.filter(owner=self.request.user)
 
     def perform_create(self, serializer):
         serializer.save(owner=self.request.user)
@@ -53,6 +57,7 @@ class NotificationEventViewSet(mixins.ListModelMixin,
 class EmailTemplateViewSet(mixins.ListModelMixin,
                            mixins.RetrieveModelMixin,
                            mixins.CreateModelMixin,
+                           mixins.UpdateModelMixin,
                            viewsets.GenericViewSet):
     """
     A ViewSet for viewing Notification Event.
@@ -92,7 +97,9 @@ class OutgoingMessageViewSet(mixins.ListModelMixin,
         return queryset.filter(Q(user=self.request.user) | Q(recipient=self.request.user.email)).distinct()
 
 
-class NotificationGroupViewSet(mixins.ListModelMixin,
+class NotificationGroupViewSet(mixins.CreateModelMixin,
+                               mixins.UpdateModelMixin,
+                               mixins.ListModelMixin,
                                mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):
     """
@@ -109,6 +116,9 @@ class NotificationGroupViewSet(mixins.ListModelMixin,
     def get_queryset(self):
         return super().get_queryset().filter(owner=self.request.user)
 
+    def perform_create(self, serializer):
+        serializer.save(owner=self.request.user)
+
 
 class SmtpProviderViewSet(mixins.ListModelMixin,
                           mixins.RetrieveModelMixin,
@@ -128,6 +138,28 @@ class SmtpProviderViewSet(mixins.ListModelMixin,
         return super().get_queryset().filter(users__id__in=[self.request.user.id]).distinct()
 
 
+class MedialViewSet(mixins.ListModelMixin,
+                    mixins.RetrieveModelMixin,
+                    mixins.CreateModelMixin,
+                    viewsets.GenericViewSet):
+    """
+    A ViewSet for viewing user uploaded media.
+    """
+    queryset = Media.objects.select_related('owner').all()
+    serializer_class = MediaSerializer
+    authentication_classes = [JWTAuthentication]
+    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
+    pagination_class = CustomLimitOffsetPagination
+    filter_backends = [filters.OrderingFilter]
+    ordering_fields = ['id']
+
+    def get_queryset(self):
+        return super().get_queryset().filter(owner=self.request.user)
+
+    def perform_create(self, serializer):
+        serializer.save(owner=self.request.user)
+
+
 class OutgoingMessageNotificationAPIView(APIView):
     parser_classes = [PlainTextParser, JSONParser]
 
