@@ -38,7 +38,7 @@ class SmtpProvider(BaseModel):
     actives = ActiveManager()
 
     def __str__(self):
-        return self.provider_name
+        return "{} - {}".format(self.id, self.provider_name)
 
     def clean(self):
         if self.email_use_ssl and self.email_use_tls:
@@ -70,7 +70,7 @@ class NotificationGroup(BaseModel):
     actives = ActiveManager()
 
     def __str__(self):
-        return self.name
+        return "{} - {}".format(self.id, self.name)
 
 
 class EmailTemplate(BaseModel):
@@ -85,7 +85,7 @@ class EmailTemplate(BaseModel):
     actives = ActiveManager()
 
     def __str__(self):
-        return self.title
+        return "{} - {}".format(self.id, self.title)
 
 
 class NotificationEvent(BaseModel):
@@ -95,11 +95,13 @@ class NotificationEvent(BaseModel):
     owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='notification_events',
                               on_delete=models.PROTECT)
     notification_group = models.ForeignKey(to=NotificationGroup, verbose_name=_('notification group'),
-                                           on_delete=models.PROTECT, related_name='notification_events')
+                                           on_delete=models.PROTECT, related_name='notification_events',
+                                           null=True, blank=True)
     template = models.ForeignKey(to=EmailTemplate, on_delete=models.PROTECT, related_name='notification_events',
                                  null=True, blank=True,
                                  help_text=_("If template is selected, HTML message and subject fields will not be used"
                                              "- they will be populated from template"))
+    recipients = models.ManyToManyField(to=User, verbose_name=_('members'), related_name='recipients', blank=True)
     subject = models.CharField(verbose_name=_('subject'), max_length=255, blank=True)
     content = models.TextField(verbose_name=_('content'), blank=True)
     is_active = models.BooleanField(verbose_name=_('is active?'), default=True)
@@ -108,13 +110,7 @@ class NotificationEvent(BaseModel):
     actives = ActiveManager()
 
     def __str__(self):
-        return self.title
-
-    def save(self, *args, **kwargs):
-        self.subject = self.subject if not self.template else ""
-        self.content = self.content if not self.template else ""
-
-        super().save(*args, **kwargs)
+        return "{} - {}".format(self.id, self.title)
 
     def get_content(self):
         return self.content if not self.template else self.template.content
@@ -122,6 +118,13 @@ class NotificationEvent(BaseModel):
     def get_subject(self):
         return self.subject if not self.template else self.template.subject
 
+    def get_recipients(self):
+        recipients = self.recipients.all()
+        if self.notification_group:
+            recipients = recipients | self.notification_group.members.all()
+
+        return recipients.filter(is_active=True, email__isnull=False).distinct()
+
 
 class Attachment(BaseModel):
     file = models.FileField(verbose_name=_('file'), upload_to='notification_system/attachment/file/',
@@ -133,6 +136,19 @@ class Attachment(BaseModel):
         return str(self.id)
 
 
+class Media(BaseModel):
+    file = models.FileField(verbose_name=_('file'), upload_to='notification_system/media/file/',
+                            validators=[validate_file_extension, validate_file_size])
+    owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='media', on_delete=models.PROTECT)
+
+    class Meta:
+        verbose_name = _('Media')
+        verbose_name_plural = _('Media')
+
+    def __str__(self):
+        return str(self.id)
+
+
 class OutgoingMessage(BaseModel):
     TYPE_EMAIL = 1
 
@@ -184,4 +200,4 @@ class OutgoingMessage(BaseModel):
                                                              default=TYPE_EMAIL)
 
     def __str__(self):
-        return self.subject
+        return "{} - {}".format(self.id, self.subject)
