@@ -4,17 +4,18 @@ from notification_system.utils import get_user_common_data
 
 
 def broadcast_event_messages_to_recipients(event_message_id):
-    notification_event = NotificationEvent.objects.prefetch_related('notification_group__members', 'attachments') \
-        .select_related('smtp_provider', 'owner', 'notification_group', 'template').get(id=event_message_id)
+    notification_event = NotificationEvent.objects.prefetch_related('notification_group__members', 'attachments')
+    notification_event = notification_event.select_related('smtp_provider', 'owner', 'notification_group', 'template')
+    notification_event = notification_event.get(id=event_message_id)
 
     subject = notification_event.get_subject()
     html_content = notification_event.get_content()
-    users = notification_event.notification_group.members.filter(is_active=True, email__isnull=False)
+    recipients = notification_event.get_recipients()
     email_from = notification_event.smtp_provider.email_from
     smtp_config = notification_event.smtp_provider.get_smtp_config()
     attachments = [attachment.file.path for attachment in notification_event.attachments.all()]
 
-    for user in users:
+    for user in recipients:
         context = get_user_common_data(user=user)
         notification_handler = NotificationHandler(subject=subject, recipient=user.email, email_from=email_from,
                                                    context=context, content=html_content, user_id=user.id,
